SuperStrict
rem
Framework brl.d3d9max2d
Import brl.random
Import brl.pngloader
Import brl.max2d
end rem


Global Main:TMain = New TMain
Main.run()

Type TMain
	Const PROG_NAME:String = "Evolution"
	Const PROG_VERSION:String = "0.9"
	'Const PROG_AUTHOR:String = "FDM"
	Const PROG_GFXWIDTH:Int = 800
	Const PROG_GFXHEIGHT:Int = 800

	Field worldProcess:TWorld_Process
	Field worldRenderer:TWorld_Renderer
	
	Field finish:Byte
	
	Method run()
		init()
		Repeat
			Cls
			worldRenderer.render()
			Flip
			worldProcess.process()
		Until finish Or AppTerminate()
	End Method
	
	Method init()
		SeedRnd MilliSecs()
		AppTitle = PROG_NAME + " " + PROG_VERSION
		Graphics PROG_GFXWIDTH, PROG_GFXHEIGHT
		New TWorld
		
		'auto-settings
		For Local i:Int = 0 Until 20
		worldProcess.smoothWater(0.5)
		Next
	End Method
End Type


Type TWorld
	Global currentWorld:TWorld
	Field location:TLocation[,]
	Field location_tmp:TLocation[,]
	Field locationWidth:Int
	Field locationHeight:Int
	Field locationSize:Int[2]
	Field lastSpawn:Int
	
	Field DnaIdCounter:Int
	Field DnaStats:TDnaStat[1]
	Field bestDNAs:TList = New TList
	Field bestMeatDNAs:TList = New TList
	
	Const VIEW_WATERSUPPORT:Byte = 1
	Const VIEW_LANDSCAPE:Byte = 2
	Const VIEW_BRAIN:Byte = 3
	Const VIEW_COUNTER:Byte = 3
	Field viewMode:Byte = VIEW_LANDSCAPE
	
	Field showBest:Byte = True
	Field showStats:Byte = True
	Field showHelp:Byte = True
	Field helpText:TList = New TList
	
	Field settingSelected:Int = 0
	Const settingMax:Int = 7
	Const SIMULATIONSTEPS:Byte = 0
	Const MUTATIONRATIO:Byte = 1
	Const PLANTGROWTH:Byte = 2
	Const MINSTEPTIME:Byte = 3
	Const MAXAGE:Byte = 4
	Const MINSCORE:Byte = 5
	Const AUTORESTART:Byte = 6
	Const AUTOSPAWN:Byte = 7
	Field SETTINGS:Float[settingMax + 1]
	Field SETTINGS_STEPWIDTH:Float[settingMax + 1]
	Field SETTINGS_MIN:Float[settingMax + 1]
	Field SETTINGS_MAX:Float[settingMax + 1]
	Field SETTINGS_NAME:String[settingMax + 1]
	
	'status vars
	Field simulatedTurns:Int
	Field creatureCount:Int
	Field oldest:Int
	Field meatEaterMax:Float
	Field meatEaterMin:Float
	Field highestGeneration:Int
	Field lowestGeneration:Int
	Field highestMutation:Int
	Field lowestMutation:Int
	Field highestPoints:Int
	Field maxScoreEver:Int
	
	Field bestOne:TCreature

	
	Method New()
		TWorld.currentWorld = Self
		'Default settings
		SETTINGS[SIMULATIONSTEPS] = 1
		SETTINGS_MIN[SIMULATIONSTEPS] = 0
		SETTINGS_MAX[SIMULATIONSTEPS] = 20
		SETTINGS_STEPWIDTH[SIMULATIONSTEPS] = 1
		SETTINGS_NAME[SIMULATIONSTEPS] = "SimulationSteps"
		
		SETTINGS[MUTATIONRATIO] = 0.3
		SETTINGS_MIN[MUTATIONRATIO] = 0
		SETTINGS_MAX[MUTATIONRATIO] = 1
		SETTINGS_STEPWIDTH[MUTATIONRATIO] = 0.01
		SETTINGS_NAME[MUTATIONRATIO] = "MutationRatio"
		
		SETTINGS[PLANTGROWTH] = 0.03
		SETTINGS_MIN[PLANTGROWTH] = 0
		SETTINGS_MAX[PLANTGROWTH] = 1
		SETTINGS_STEPWIDTH[PLANTGROWTH] = 0.001
		SETTINGS_NAME[PLANTGROWTH] = "PlantGrowth"
		
		SETTINGS[MINSTEPTIME] = 0
		SETTINGS_MIN[MINSTEPTIME] = 0
		SETTINGS_MAX[MINSTEPTIME] = 1000
		SETTINGS_STEPWIDTH[MINSTEPTIME] = 10
		SETTINGS_NAME[MINSTEPTIME] = "MinStepTime"
		
		SETTINGS[MAXAGE] = 500
		SETTINGS_MIN[MAXAGE] = 10
		SETTINGS_MAX[MAXAGE] = 2323230
		SETTINGS_STEPWIDTH[MAXAGE] = 10
		SETTINGS_NAME[MAXAGE] = "MaxAge"
		
		SETTINGS[MINSCORE] = 50
		SETTINGS_MIN[MINSCORE] = 0
		SETTINGS_MAX[MINSCORE] = 232323
		SETTINGS_STEPWIDTH[MINSCORE] = 10
		SETTINGS_NAME[MINSCORE] = "MinScore"
		
		SETTINGS[AUTORESTART] = 1
		SETTINGS_MIN[AUTORESTART] = 0
		SETTINGS_MAX[AUTORESTART] = 1
		SETTINGS_STEPWIDTH[AUTORESTART] = 1
		SETTINGS_NAME[AUTORESTART] = "AutoRestart"
		
		SETTINGS[AUTOSPAWN] = 1
		SETTINGS_MIN[AUTOSPAWN] = 0
		SETTINGS_MAX[AUTOSPAWN] = 100
		SETTINGS_STEPWIDTH[AUTOSPAWN] = 1
		SETTINGS_NAME[AUTOSPAWN] = "AutoSpawn"
		
		
		
		
		'create modell
		locationSize[0] = 8
		locationSize[1] = 8
		locationWidth = Main.PROG_GFXWIDTH / locationSize[0]
		locationHeight = Main.PROG_GFXHEIGHT / locationSize[1]
		location = New TLocation[locationWidth, locationHeight]
		For Local w:Int = 0 Until locationWidth
		For Local h:Int = 0 Until locationHeight
			location[w, h] = New TLocation
			location[w, h].calcGround()
		Next
		Next
		
		bestOne = New TCreature
		bestOne.dnaId = getNewDnaId()
		
		helpText.AddLast("F1 - toggle this text")
		helpText.AddLast("F2 - toggle stats")
		helpText.AddLast("F3 - toggle best")
		helpText.AddLast("S - smooth landscape")
		helpText.AddLast("C - create 1 creature")
		helpText.AddLast("X - kill all creatures")
		helpText.AddLast("V - change view")
		helpText.AddLast("Left/Right - +- setting value")
		helpText.AddLast("Up/Down - change setting mode")
		helpText.AddLast("Spacebar - create 50 creatures")
		helpText.AddLast("Left Mouse - create best creature")
		helpText.AddLast("Right Mouse - let it rain!")
		helpText.AddLast("N - load best creature")
		helpText.AddLast("M - save best creature")
		
		'create view
		Local renderer:TWorld_Renderer = New TWorld_Renderer
		renderer.world = Self
		Main.worldRenderer = renderer
		
		'create controll
		Local process:TWorld_Process = New TWorld_Process
		process.world = Self
		Main.worldProcess = process
	End Method
	
	Method getNewDnaId:Int()
		Const INCAMOUNT:Int = 50
		DnaIdCounter:+1
		If DnaStats.Length <= DnaIdCounter
			DnaStats = DnaStats[..(DnaStats.length + INCAMOUNT)]
			For Local i:Int = 0 Until INCAMOUNT
				DnaStats[DnaStats.length + i - INCAMOUNT] = New TDnaStat
			Next
		EndIf
		Return DnaIdCounter
	End Method
	
	Method checkDNARanks(c:TCreature, dnalist:TList)
		Const MAXCOUNT:Int = 10
		Local okay:Byte = False
		Local counter:Int = 0
		For Local c2:TCreature = EachIn dnalist
			If c2.dnaId = c.dnaId
				dnalist.Sort(False, compareDNAs)
				Return
			EndIf
			If DnaStats[c.dnaId].getAvgScore() > DnaStats[c2.dnaId].getAvgScore() okay = True
			counter:+1
		Next
		If counter < MAXCOUNT okay = True
		If Not okay Return
		If c.score = 0 Return
		dnalist.AddLast(c)
		dnalist.Sort(False, compareDNAs)
		While dnalist.Count() > MAXCOUNT
				dnalist.RemoveLast()
		Wend
	End Method
	
	Function compareDNAs:Int(o1:Object, o2:Object)
		Local c1:TCreature = TCreature(o1)
		Local c2:TCreature = TCreature(o2)
		If c1 = Null Or c2 = Null Return Null
		Local a:Float = currentWorld.DnaStats[c1.dnaId].getAvgScore()
		Local b:Float = currentWorld.DnaStats[c2.dnaId].getAvgScore()
		Local ac:Int = currentWorld.DnaStats[c1.dnaId].counter
		Local bc:Int = currentWorld.DnaStats[c2.dnaId].counter
		
		If a > b
			Return 1
		ElseIf a < b
			Return - 1
		ElseIf ac > bc
			Return 1
		ElseIf ac < bc
			Return - 1
		Else
			Return 0
		End If
	End Function
	
	Method getLocation:TLocation(w:Int, h:Int)
		w:Mod locationWidth
		h:Mod locationHeight
		If w < 0 w:+locationWidth
		If h < 0 h:+locationHeight
		Return location[w, h]
	End Method
	
	Method getTempLocation:TLocation(w:Int, h:Int)
		w:Mod locationWidth
		h:Mod locationHeight
		If w < 0 w:+locationWidth
		If h < 0 h:+locationHeight
		Return location_tmp[w, h]
	End Method
	
	Method GetWHByXY(x:Float, y:Float, w:Int Var, h:Int Var)
		w = Int(x / locationSize[0])
		h = Int(y / locationSize[1])
		w:Mod locationWidth
		h:Mod locationHeight
		If w < 0 w:+locationWidth
		If h < 0 h:+locationHeight
	End Method
	
	Const MAGICNUMBER:Int = 23
	
	Method saveBest()
		Local path:String = RequestFile("Select sce file to save", "Creature Files:cre;All Files:*", True, AppDir + "/champion")
		If path = Null Return
		Local file:TStream = WriteFile(path)
		WriteInt(file, MAGICNUMBER)
		WriteInt(file, bestOne.score)
		WriteFloat(file, bestOne.meatEater)
		WriteFloat(file, bestOne.metabolism)
		For Local y:Int = 0 Until TCreature.OUTPUTCOUNT
			For Local x:Int = 0 Until TCreature.INPUTCOUNT
				WriteFloat(file, bestOne.transform[x, y])
			Next
		Next
		CloseFile file
	End Method
	
	Method loadBest()
		Local path:String = RequestFile("Select sce file to open", "Creature Files:cre;All Files:*", False, AppDir + "/champion")
		If path = Null Return
		Local file:TStream = ReadFile(path)
		If file = Null Return
		bestOne = New TCreature
		If ReadInt(file) <> MAGICNUMBER Return
		bestOne.score = ReadInt(file)
		bestOne.meatEater = ReadFloat(file)
		bestOne.metabolism = ReadFloat(file)
		For Local y:Int = 0 Until TCreature.OUTPUTCOUNT
			For Local x:Int = 0 Until TCreature.INPUTCOUNT
			bestOne.transform[x, y] = ReadFloat(file)
			Next
		Next
		bestOne.dnaId = getNewDnaId()
		CloseFile file
	End Method
End Type

Type TWorld_Renderer
	Field world:TWorld
	
	Field grasImg:TImage
	Field groundImg:TImage
	Field desertImg:TImage
	Field waterImg:TImage
	Field creatureImg:TImage
	Field rectImg:TImage
	Field bloodImg:TImage
	
	Field font:TImageFont
	
	Method New()
		grasImg = LoadAnimImage("data\grass.png", 8, 8, 0, 100)
		'groundImg = LoadAnimImage("data\ground.png", 8, 8, 0, 100)
		groundImg = LoadImage("data\ground.png")
		desertImg = LoadAnimImage("data\desert.png", 8, 8, 0, 100)
		waterImg = LoadImage("data\water.png")
		creatureImg = LoadImage("data\creature.png")
		rectImg = LoadImage("data\rect.png")
		bloodImg = LoadImage("data\blood.png")
		font = LoadImageFont("data\type_writer.ttf", 8, SMOOTHFONT)
		SetImageFont(font)
	End Method
	
	Method renderMap()
		If world.viewMode = world.VIEW_LANDSCAPE
			sc 255, 255, 255
			TileImage groundImg, 0, 0
		EndIf
		
		For Local w:Int = 0 Until world.locationWidth
		For Local h:Int = 0 Until world.locationHeight
			Local l:TLocation = world.location[w, h]
			Local posx:Int = w * world.locationSize[0]
			Local posy:Int = h * world.locationSize[1]
			
			'show water support
			Select world.viewMode
				Case world.VIEW_WATERSUPPORT
					Local s:Float = (l.waterSupport + 1.0) / 2
					sc 0, 100 * s, 200 * s
					DrawImage rectImg, posx, posy
				Case world.VIEW_LANDSCAPE
					If l.ground = TLocation.GR_DESERT
						SetAlpha - (l.baseWaterSupport + 0.5) * 0.5
						sc 255, 255, 255
						DrawImage desertImg, posx, posy, l.randval
					ElseIf l.ground = TLocation.GR_GRAS
						sc 255, 255, 255
						'DrawImage groundImg, posx, posy, l.randval
					ElseIf l.ground = TLocation.GR_WATER
						Local b:Float = (Sin(((l.randval + MilliSecs() * 0.1) Mod 100) * 3.6)) * 16
						Local a:Float = (1 - l.baseWaterSupport) * 128 + 128 + b
						sc a, a, a
						DrawImage waterImg, posx, posy
					EndIf
					
					If l.plants > 0
						SetAlpha l.plants
						SetColor 255, 255, 255
						DrawImage grasImg, posx, posy, l.randval
						SetAlpha 1
					End If
					
					If l.blood > 0.1
						SetAlpha l.blood
						SetColor 255, 0, 0
						DrawImage bloodImg, posx, posy
						SetAlpha 1
					End If
			End Select
			SetAlpha 1
			
			
			Local c:TCreature = world.location[w, h].creature
			If c
				sc 250, 50 + 150 * (1 - c.meatEater), 50
				DrawImage creatureImg, posx, posy
				'DrawRect w * world.locationSize[0] + 1, h * world.locationSize[1] + 1, world.locationSize[0] - 3, world.locationSize[1] - 3
			End If
			'sc 0, 0, 0
			'DrawText Floor(world.location[w, h].WaterSupport * 100), w * world.locationSize[0], h * world.locationSize[1]	
		Next
		Next
	End Method
	
	Method renderBestBrain()
		Local c:TCreature = world.bestOne
		Local xi:Int = GraphicsWidth() / 6, yi:Int = GraphicsHeight() / 12
		Local xo:Int = xi + 4 * GraphicsWidth() / 6, yo:Int = yi
		Local dyi:Int = 5 * GraphicsWidth() / (6 * c.INPUTCOUNT)
		Local dyo:Int = 5 * GraphicsWidth() / (6 * c.OUTPUTCOUNT)
		Local rectsize:Int = 10
		Local rectsize2:Int = rectsize / 2
		
		
		SetBlend LIGHTBLEND
		
		'calc norm for every output
		Local norm:Float[c.OUTPUTCOUNT]
		
		
		If c = Null Return 
		For Local op:Int = 0 Until c.OUTPUTCOUNT
			For Local ip:Int = 0 Until c.INPUTCOUNT
				'norm[op]:+c.transform[ip, op] * c.transform[ip, op]
				norm[op] = Max(norm[op], Abs(c.transform[ip, op]))
			Next
			'norm[op] = Sqr(norm[op])
		Next
		
		
		For Local ip:Int = 0 Until c.INPUTCOUNT
		For Local op:Int = 0 Until c.OUTPUTCOUNT
			Local tr:Float = c.transform[ip, op] / norm[op]
			If tr > 0
				sc 0, 255 * tr, 0
			ElseIf tr < 0
				sc - 255 * tr, 0, 0
			EndIf
			if tr <> 0 DrawLine xi, yi + dyi * ip, xo, yo + dyo * op
		Next
		Next
		SetAlpha 1
		
		sc 255, 255, 255
		For Local ip:Int = 0 Until c.INPUTCOUNT
			DrawRect xi - rectsize2, yi + dyi * ip - rectsize2, rectsize, rectsize
		Next
		sc 255, 255, 255
		For Local op:Int = 0 Until c.OUTPUTCOUNT
			DrawRect xo - rectsize2, yo + dyo * op - rectsize2, rectsize, rectsize
		Next
			
		SetBlend ALPHABLEND
	End Method
	
	Method render()
		Select world.viewMode
			Case world.VIEW_WATERSUPPORT
				renderMap()
			Case world.VIEW_LANDSCAPE
				renderMap()
			Case world.VIEW_BRAIN
				renderBestBrain()
		End Select

		If world.showHelp
			SetBlend ALPHABLEND
			Local y:Int = 0
			Local x:Int = 0
			sc 0, 0, 0
			SetAlpha 0.5
			DrawRect x, y, 200, 14 * world.helpText.Count() + 15
			x:+5
			y:+10
			For Local s:String = EachIn world.helpText
				sc 100, 255, 100
				dt(s, x, y)
				y:+14
			Next
		End If
		
		If world.showStats
			Local x:Int = 200
			Local y:Int = 0
			Local yd:Int = 10
			sc 0, 0, 0
			SetAlpha 0.5
			DrawRect x, y, 160, (10 + world.settingMax) * yd + 15
			SetAlpha 1
			x:+5
			y:+10
			'DrawText "Selected Setting: " + world.SETTINGS_NAME[world.settingSelected], x, y ;y:+yd
			For Local i:Int = 0 To world.settingMax
				sc 100, 100, 255
				If world.settingSelected = i sc 255, 100, 100
				dt world.SETTINGS_NAME[i] + ": " + world.SETTINGS[i], x, y ;y:+yd
			Next
			
			y:+yd
			sc 255, 255, 100
			dt "Turn: " + world.simulatedTurns, x, y;y:+yd
			dt "Creatures: " + world.creatureCount, x, y;y:+yd
			dt "Oldest: " + world.oldest, x, y;y:+yd
			dt "MeatMax: " + Percent(world.meatEaterMax), x, y;y:+yd
			dt "MeatMin: " + Percent(world.meatEaterMin), x, y;y:+yd
			
			dt "Min/MaxGeneration: " + Int(world.lowestGeneration) + "/" + Int(world.highestGeneration), x, y;y:+yd
			dt "Min/MaxMutation: " + Int(world.lowestMutation) + "/" + Int(world.highestMutation), x, y;y:+yd
			dt "MaxScore(Cur|Ever): " + Int(world.highestPoints) + " | " + Int(world.maxScoreEver), x, y;y:+yd
		End If
		
		'show best
		If world.showBest
			Local x:Int = 360
			Local y:Int = 0
			sc 0, 0, 0
			SetAlpha 0.5
			DrawRect x, y, 440, 12 * world.bestDNAs.Count() + 20 + 12
			SetAlpha 1
			x:+10
			y:+15
			sc 255, 255, 100
			dt "All Creatures Top 10", x, y
			y:+12
			For Local c:TCreature = EachIn world.bestDNAs
				sc 255, 255, 255
				dt "Id = " + c.dnaId + "(" + world.DnaStats[c.dnaId].counter + ") score: " + world.DnaStats[c.dnaId].getAvgScore(), x, y
				y:+12
			Next
			y = 15
			x = 360 + 220 + 20
			sc 255, 100, 100
			dt "Meat Eaters Top 10", x, y
			y:+12
			For Local c:TCreature = EachIn world.bestMeatDNAs
				sc 255, 255, 255
				dt "Id = " + c.dnaId + "(" + world.DnaStats[c.dnaId].counter + ") score: " + world.DnaStats[c.dnaId].getAvgScore(), x, y
				y:+12
			Next
		EndIf
	End Method
End Type

Function Percent:String(a:Float)
	Return Int(a * 100) + "%"
End Function

Type TWorld_Process
	Field world:TWorld
	Field lastStep:Int
	
	Method process()
		If (MilliSecs() - lastStep) >= world.SETTINGS[world.MINSTEPTIME]
			For Local s:Int = 0 Until world.SETTINGS[world.SIMULATIONSTEPS]
				world.simulatedTurns:+1
				world.location_tmp = New TLocation[world.locationWidth, world.locationHeight]
				For Local w:Int = 0 Until world.locationWidth
				For Local h:Int = 0 Until world.locationHeight
					world.location_tmp[w, h] = world.location[w, h].copy()
					If world.location_tmp[w, h].creature world.location_tmp[w, h].creature.thought = False
				Next
				Next
				
				world.creatureCount = 0
				world.oldest = 0
				world.meatEaterMax = 0
				world.meatEaterMin = 1
				world.lowestGeneration = world.highestGeneration
				world.highestGeneration = 0
				world.highestPoints = 0
				world.lowestMutation = world.highestMutation
				world.highestMutation = 0
				
				
				For Local w:Int = 0 Until world.locationWidth
				For Local h:Int = 0 Until world.locationHeight
					Local l:TLocation = world.location_tmp[w, h]
					processLocation(l, w, h)
					If l.creature
						world.creatureCount:+1
						processCreature(l, w, h)
					End If
				Next
				Next
				world.location = world.location_tmp
			Next
			lastStep = MilliSecs()
		EndIf
		
		If KeyHit(KEY_ESCAPE) Main.finish = True
		
		If KeyHit(KEY_C) CreateRandomCreature()
		If KeyHit(KEY_X) KillAllCreatures()
		If KeyHit(KEY_F1) world.showHelp = Not world.showHelp
		If KeyHit(KEY_F2) world.showStats = Not world.showStats
		If KeyHit(KEY_F3) world.showBest = Not world.showBest
		
		If KeyHit(KEY_S) smoothWater(0.5)
		If KeyHit(KEY_V) world.viewMode = world.viewMode Mod (world.VIEW_COUNTER) + 1
		If KeyHit(KEY_M) world.saveBest()
		If KeyHit(KEY_N) world.loadBest()
		

		
		If KeyHit(KEY_UP)
			world.settingSelected:-1
			If world.settingSelected < 0 world.settingSelected = world.settingMax
		EndIf
		If KeyHit(KEY_DOWN) world.settingSelected:+1; world.settingSelected:Mod (world.settingMax + 1)
		
		Local changeSetting:Int = 0
		If KeyHit(KEY_LEFT) changeSetting = -1
		If KeyHit(KEY_RIGHT) changeSetting = + 1
		
		If changeSetting
			world.SETTINGS[world.settingSelected]:+world.SETTINGS_STEPWIDTH[world.settingSelected] * changeSetting
			If world.SETTINGS[world.settingSelected] < world.SETTINGS_MIN[world.settingSelected] world.SETTINGS[world.settingSelected] = world.SETTINGS_MIN[world.settingSelected]
			If world.SETTINGS[world.settingSelected] > world.SETTINGS_MAX[world.settingSelected] world.SETTINGS[world.settingSelected] = world.SETTINGS_MAX[world.settingSelected]
		End If
		
		If KeyDown(KEY_SPACE) Or (world.SETTINGS[TWorld.AUTORESTART] And world.creatureCount = 0)
			For Local i:Int = 0 Until 50
			CreateRandomCreature()
			Next
		EndIf
		
		If world.SETTINGS[TWorld.AUTOSPAWN] And MilliSecs() > Self.world.lastSpawn
			For Local i:Int = 0 Until world.SETTINGS[TWorld.AUTOSPAWN]
			CreateRandomCreature()
			Next
			Self.world.lastSpawn = MilliSecs() + 1000 / world.SETTINGS[TWorld.AUTOSPAWN]
		EndIf
		
		
		Local mw:Int, mh:Int
		world.GetWHByXY(MouseX(), MouseY(), mw, mh)
		
		If MouseHit(MOUSE_LEFT)
			CreateBestCreature(mw, mh)
		EndIf
		
		If MouseDown(MOUSE_RIGHT)
			createRain(mw, mh, 5)
		End If
	End Method
	
	Method createRain(w:Int, h:Int, r:Int)
		For Local x:Int = -r To r
		For Local y:Int = -r To r
			Local lo:TLocation = world.getLocation(w + x, h + y)
			lo.WaterSupport:+Max(0, (1 - Sqr(x * x + y * y) / Sqr(r * r))) * 10
		Next
		Next
	End Method
	
	Method killAllCreatures()
		For Local w:Int = 0 Until world.locationWidth
		For Local h:Int = 0 Until world.locationHeight
			world.location_tmp[w, h].creature = Null
			world.location[w, h].creature = Null
		Next
		Next
	End Method
	
	Method CreateRandomCreature(w:Int = -1, h:Int = -1)
			If w < 0 w = Rand(0, world.locationWidth - 1)
			If h < 0 h = Rand(0, world.locationHeight - 1)
			Local c:TCreature
			Local p:Float = Rnd(0, 1)
			
			If p >= 0.4
				Local sel:Int = Rand(0, world.bestDNAs.count() - 1)
				For Local c2:TCreature = EachIn world.bestDNAs
					sel:-1
					If sel = 0
						c = c2.child()
						Exit
					End If
				Next
				If c mutateCreature(c)
			ElseIf p >= 0.1
				Print "create meateater"
				Local sel:Int = Rand(0, world.bestMeatDNAs.count() - 1)
				For Local c2:TCreature = EachIn world.bestMeatDNAs
					sel:-1
					If sel = 0
						c = c2.child()
						Exit
					End If
				Next
				If c = Null
					c = New TCreature
					c.meatEater = 1
					c.dnaId = world.getNewDnaId()
				Else
					mutateCreature(c)
				EndIf
			EndIf
			
			If p < 0.1 Or c = Null
				c = New TCreature
				c.dnaId = world.getNewDnaId()
			EndIf
			
			While addCreature(c, w, h) = Null
				w:+1
				If (w Mod world.locationWidth) = 0 h:+1
			Wend
	End Method
	
	Method CreateBestCreature(w:Int = -1, h:Int = -1)
			If w < 0 w = Rand(0, world.locationWidth - 1)
			If h < 0 h = Rand(0, world.locationHeight - 1)
			Local c:TCreature = world.bestOne.child()
			While addCreature(c, w, h) = Null
				w:+1
				If (w Mod world.locationWidth) = 0 h:+1
			Wend
	End Method
	
	Method addCreature:TCreature(c:TCreature, w:Int, h:Int)
		Local l:TLocation = world.getLocation(w, h)
		If l.creature Or l.ground = TLocation.GR_WATER Return Null
		l.Creature = c
		Return l.Creature
	End Method
	
	Method addCreatureToTemp:TCreature(c:TCreature, w:Int, h:Int)
		Local l:TLocation = world.getTempLocation(w, h)
		If l.creature Or l.ground = TLocation.GR_WATER Return Null
		l.Creature = c
		Return l.Creature
	End Method
	
	Method processLocation(l:TLocation, w:Int, h:Int)
		flowWater(l, w, h)
		l.blood = l.blood * 0.99
		If l.WaterSupport > 0 And l.ground = TLocation.GR_GRAS And l.plants < 1
			l.plants:+world.SETTINGS[TWorld.PLANTGROWTH] * l.WaterSupport
			'l.WaterSupport:-0.1
			l.WaterSupport = 0
			If l.plants > 1 l.plants = 1
		End If
	End Method
	
	Method flowWater(l:TLocation, w:Int, h:Int)
			Local waterSupport:Float = 0
			Local p:Float = 0.001
			
			For Local w2:Int = w - 1 To w + 1
			For Local h2:Int = h - 1 To h + 1
				If (Abs(w2 - w) + Abs(h2 - h)) = 1
					Local lo2:TLocation = world.getLocation(w2, h2)
					waterSupport:+lo2.waterSupport
				EndIf
			Next
			Next
			l.waterSupport = ((l.baseWaterSupport + l.waterSupport) * (1 - p) / 2 + waterSupport * (p) / 5)
	End Method
	
	Method processCreature(l:TLocation, w:Int, h:Int)
		If l.creature = Null Return
		Local c:TCreature = l.creature
			world.oldest:Int = Max(world.oldest, c.age)
			world.meatEaterMax:Float = Max(world.meatEaterMax, c.meatEater)
			world.meatEaterMin:Float = Min(world.meatEaterMin, c.meatEater)
			world.highestGeneration = Max(world.highestGeneration, c.generation)
			world.lowestGeneration = Min(world.lowestGeneration, c.generation)
			world.highestPoints = Max(world.highestPoints, c.score)
			world.highestMutation = Max(world.highestMutation, c.mutation)
			world.lowestMutation = Min(world.lowestMutation, c.mutation)
			world.maxScoreEver = Max(world.maxScoreEver, c.score)
		thinkCreature(l, w, h)
	End Method
	
	Method mutateCreature(c:TCreature)
		If Rnd(0, 1) > world.SETTINGS[TWorld.MUTATIONRATIO] Return
		c.mutation:+1
		If Rnd(0, 1) < world.SETTINGS[TWorld.MUTATIONRATIO] c.meatEater = Rand(1, 10) / 10.0
		'c.metabolism:*Rnd(0.9, 1.1)
		c.mutateKi(world.SETTINGS[TWorld.MUTATIONRATIO])
		c.dnaId = world.getNewDnaId()
	End Method
	
	Method thinkCreature(l:TLocation, w:Int, h:Int)
		Local c:TCreature = l.creature
		If c.thought Return
		c.thought = True
		c.hungry[0]:+c.metabolism * (1 + 0.5 * (c.moveTo > 0))
		c.hungry[1]:+c.metabolism * (1 + 0.1 * (c.moveTo > 0))
		c.sex:+c.metabolism
		c.age:+1
		c.moveTo = 0
		
		If c.sex > 1 And c.score >= world.SETTINGS[TWorld.MINSCORE] And c.outputs[6]
			Local c2:TCreature
			Local i:Int = Rand(1, 2)
			For Local i2:Int = 0 Until i
				c2 = addCreatureToTemp(c.child(), w + Rand(-1, 1), h + Rand(-1, 1))
				If c2 mutateCreature(c2)
			Next
			c.sex = 0
		End If
		
		Local lo2:TLocation
		
		c.inputs[5] = l.plants
		c.inputs[22] = l.waterSupport
		lo2 = world.getTempLocation(w, h - 1)
		c.inputs[1] = lo2.ground <> TLocation.GR_WATER And lo2.creature = Null
		c.inputs[6] = lo2.plants
		c.inputs[23] = lo2.waterSupport
		lo2 = world.getTempLocation(w + 1, h)
		c.inputs[2] = lo2.ground <> TLocation.GR_WATER And lo2.creature = Null
		c.inputs[7] = lo2.plants
		c.inputs[24] = lo2.waterSupport
		lo2 = world.getTempLocation(w, h + 1)
		c.inputs[3] = lo2.ground <> TLocation.GR_WATER And lo2.creature = Null
		c.inputs[8] = lo2.plants
		c.inputs[25] = lo2.waterSupport
		lo2 = world.getTempLocation(w - 1, h)
		c.inputs[4] = lo2.ground <> TLocation.GR_WATER And lo2.creature = Null
		c.inputs[9] = lo2.plants
		c.inputs[26] = lo2.waterSupport
		
		c.inputs[10] = creatureInReach(c, w, h, 1)
		c.inputs[11] = creatureInReach(c, w, h - 1, 1)
		c.inputs[12] = creatureInReach(c, w + 1, h, 1)
		c.inputs[13] = creatureInReach(c, w, h + 1, 1)
		c.inputs[14] = creatureInReach(c, w - 1, h, 1)
		
		c.inputs[15] = c.meatEater
		
		c.inputs[16] = c.outputs[0]
		c.inputs[17] = c.outputs[1]
		c.inputs[18] = c.outputs[2]
		c.inputs[19] = c.outputs[3]
		c.inputs[20] = c.outputs[4]
		c.inputs[21] = c.outputs[5]
		
		
		
		'output to input loop
		For Local i:Int = 1 Until TCreature.LOOPCOUNT
			c.inputs[TCreature.INPUTCOUNT - TCreature.LOOPCOUNT + i] = c.outputs[TCreature.OUTPUTCOUNT - TCreature.LOOPCOUNT + i]
		Next
		
		c.calcKI()
		
		Local val:Float = 0
		c.moveTo = 0
		If c.outputs[0] > val val = c.outputs[0] ;c.moveTo = 1
		If c.outputs[1] > val val = c.outputs[1] ;c.moveTo = 2
		If c.outputs[2] > val val = c.outputs[2] ;c.moveTo = 3
		If c.outputs[3] > val val = c.outputs[3] ;c.moveTo = 4
		
		l = moveCreature(l, w, h)
		
		If c.outputs[4] >= 1 eatCreature(l, w, h)
		If c.outputs[5] >= 1 eatPlant(l, w, h)
		
		If l.ground = TLocation.GR_WATER dieCreature(l)
		
		If (c.hungry[0] * (1 - c.meatEater) + c.hungry[1] * c.meatEater) > 1 dieCreature(l)
		If c.age > world.SETTINGS[TWorld.MAXAGE] dieCreature(l)
	End Method
	
	Method dieCreature(l:TLocation)
		Local c:TCreature = l.creature
		If c = Null Return
		world.DnaStats[c.dnaId].addScore(c.score)
		l.creature = Null
		
		world.checkDNARanks(c, world.bestDNAs)
		'If c.meatEater > 0.5 And c.score DebugStop()
		
		If c.meatEater > 0.5 world.checkDNARanks(c, world.bestMeatDNAs)
		world.bestOne = TCreature(world.bestDNAs.First())
	End Method
	
	Method creatureInReach:Float(own:TCreature, targetw:Int, targeth:Int, r:Int)
		Local h:Float = 0
		For Local x:Int = -r To r
		For Local y:Int = -r To r
			If (Abs(x) + Abs(y) < r)
				Local loc:TLocation = world.getTempLocation(targetw + x, targeth + y)
				If loc.creature And loc.creature <> own
					h:+own.meatEater - loc.creature.meatEater
				EndIf
			EndIf
		Next
		Next
		Return h
	End Method
	
	Method eatPlant(l:TLocation, w:Int, h:Int)
		Local c:TCreature = l.creature
		If c = Null Return
		If c.meatEater = 1 Return
		If l.plants < 1 Or c.hungry[0] < 0.25 Return
		l.plants = 0
		c.hungry[0] = c.hungry[0] * c.meatEater
		c.score:+10 * (1 - c.meatEater)
	End Method
	
	Method eatCreature(l:TLocation, w:Int, h:Int)
		Local c:TCreature = l.creature
		If c.meatEater = 0 Return
		If c.hungry[1] > 0.25 Return
		If huntCreature(l, w, h)
			c.hungry[1]:-c.hungry[1] * c.meatEater * 5
			c.score:+10 * c.meatEater
		EndIf
	End Method
	
	Method huntCreature:Byte(l:TLocation, w:Int, h:Int)
		Local lo2:TLocation
		
		For Local x:Int = -1 To 1
			For Local y:Int = -1 To 1
				lo2 = world.getTempLocation(w + x, h + y)
				If lo2.creature And lo2.creature <> l.creature Exit
			Next
			If lo2.creature And lo2.creature <> l.creature Exit
		Next
		
		If lo2.creature = Null Return False
		If lo2.creature = l.creature Return False
		
		Local fight:Float = Rand(0, l.creature.meatEater) - Rand(0, lo2.creature.meatEater)
		
		If fight < 0
			l.blood = 1
			dieCreature(l)
			Return False
		ElseIf fight = 0
			Return False
		Else
			lo2.blood = 1
			dieCreature(lo2)
			Return True
		EndIf
	End Method
	
	Method moveCreature:TLocation(l:TLocation, w:Int, h:Int)
		Local c:TCreature = l.creature
		If c = Null Return Null
		Local lo2:TLocation
			
		Select c.moveTo
			Case 0
				lo2 = world.getTempLocation(w, h)
			Case 1'right
				lo2 = world.getTempLocation(w, h - 1)
			Case 2
				lo2 = world.getTempLocation(w + 1, h)
			Case 3
				lo2 = world.getTempLocation(w, h + 1)
			Case 4
				lo2 = world.getTempLocation(w - 1, h)
		End Select
		
		If lo2.creature = Null And lo2.ground <> TLocation.GR_WATER
			lo2.creature = l.creature
			l.creature = Null
			Return lo2
		EndIf
		Return l
	End Method
	
	
	Method smoothWater(p:Float = 0.5)
		world.location_tmp = New TLocation[world.locationWidth, world.locationHeight]
		
		Local minWater:Float, maxWater:Float
		For Local w:Int = 0 Until world.locationWidth
		For Local h:Int = 0 Until world.locationHeight
			world.location_tmp[w, h] = world.location[w, h].copy()
			Local l:TLocation = world.location_tmp[w, h]
			
			Local baseWaterSupport:Float = 0
			
			For Local w2:Int = w - 1 To w + 1
			For Local h2:Int = h - 1 To h + 1
				If (Abs(w2 - w) + Abs(h2 - h)) = 1
					Local lo2:TLocation = world.getLocation(w2, h2)
					baseWaterSupport:+lo2.baseWaterSupport
				EndIf
			Next
			Next
			
			l.baseWaterSupport = (l.baseWaterSupport * (1 - p) + (baseWaterSupport * (p) / 4.0))
			minWater = Min(l.baseWaterSupport, minWater)
			maxWater = Max(l.baseWaterSupport, maxWater)
		Next
		Next
		'normalize to +- 1
		Local scaleWater:Float = 2.0 / (maxWater - minWater)
		For Local w:Int = 0 Until world.locationWidth
		For Local h:Int = 0 Until world.locationHeight
			Local l:TLocation = world.location_tmp[w, h]
			l.baseWaterSupport = (l.baseWaterSupport - minWater) * scaleWater - 1
			l.calcGround()
		Next
		Next
		world.location = world.location_tmp
	End Method
End Type

Type TLocation
	'gras, water, desert
	Field baseWaterSupport:Float
	Field WaterSupport:Float
	
	Const GR_WATER:Byte = 1
	Const GR_GRAS:Byte = 2
	Const GR_DESERT:Byte = 3
	Field ground:Byte
	Field randval:Int
	Field blood:Float
	
	Field creature:TCreature
	Field plants:Float

	Method New()
		baseWaterSupport = Rand(-1, 1)
		WaterSupport = baseWaterSupport
		randval = Rand(0, 99)
	End Method
	
	Method copy:TLocation()
		Local l:TLocation = New TLocation
		l.baseWaterSupport = baseWaterSupport
		l.WaterSupport = WaterSupport
		l.ground = ground
		l.plants = plants
		l.randval = randval
		l.blood = blood
		If creature l.creature = creature.copy()
		Return l
	End Method
	
	Method calcGround()
		If baseWaterSupport < - 0.5
			ground = GR_DESERT
			plants = 0
		ElseIf baseWaterSupport < 0.5
			ground = GR_GRAS
		Else
			ground = GR_WATER
			plants = 0
		EndIf
	End Method
End Type


Type TCreature
	Field moveTo:Byte '0 not, 1=up, 2=right, 3=down, 4=left
	
	Field hungry:Float[2]
	Field age:Float = 0.0
	Field sex:Float = 0
	
	Field meatEater:Float
	Field metabolism:Float
	
	Field thought:Byte
	Field score:Int
	Field generation:Int
	Field mutation:Int
	
	Field dnaId:Int
	
	Const LOOPCOUNT:Int = 20
	
	Const INPUTCOUNT:Int = 27 + LOOPCOUNT
	'1 - walkable up
	'2 - walkable right
	'3 - walkable down
	'4 - walkable left
	'5 - plants now
	'6 - plants up
	'7 - plants right
	'8 - plants down
	'9 - plants left
	'10 - isCreature inReach now
	'11 - isCreature inReach up
	'12 - isCreature inReach right
	'13 - isCreature inReach down
	'14 - isCreature inReach left
	'15 - meatEater 
	'16 - old: went up
	'17 - old: went right
	'18 - old: went down
	'19 - old: went left
	'20 - old: hunt
	'21 - old: eat Gras
	'22 - water now
	'23 - water up
	'23 - water right
	'23 - water down
	'23 - water left
	'22.. output loop
	Const OUTPUTCOUNT:Int = 6 + LOOPCOUNT
		'0 - go up
		'1 - go right
		'2 - go down
		'3 - go left
		'4 - hunt
		'5 - eat Gras
		'6 - have sex
		'7.. - undefined for input return loop
	Field inputs:Float[TCreature.INPUTCOUNT]
	Field outputs:Float[TCreature.OUTPUTCOUNT]
	Field transform:Float[TCreature.INPUTCOUNT, TCreature.OUTPUTCOUNT]'if size length(inputs) * length(outputs) (x * y)
	
	Method New()
		meatEater = Rand(0, 1) * Rand(0, 10) / 10.0
		metabolism = 0.05
		inputs[0] = - 1
		mutateKi(1)
	End Method
	
	Method mutateKi(p:Float = 0.5)
		For Local y:Int = 0 Until TCreature.OUTPUTCOUNT
		For Local x:Int = 0 Until TCreature.INPUTCOUNT
			If Rnd(0, 1) < p
				transform[x, y]:+Rnd(-p, p)
				If Rnd(0, 1) < p transform[x, y]:*Rand(-1, 1)
			EndIf
		Next
		Next
	End Method
	
	Method calcKI()
		For Local y:Int = 0 Until TCreature.OUTPUTCOUNT
			outputs[y] = 0
			For Local x:Int = 0 Until TCreature.INPUTCOUNT
				outputs[y]:+transform[x, y] * inputs[x]
			Next
			'outputs[y] = (2 / (1 + Exp(outputs[y]))) - 1
			
			outputs[y] = Sgn(outputs[y])
		Next
	End Method

	Method copy:TCreature()
		Local c:TCreature = New TCreature
		c.moveTo = moveTo
		c.hungry[0] = hungry[0]
		c.hungry[1] = hungry[1]
		c.meatEater = meatEater
		c.metabolism = metabolism
		c.thought = thought
		c.score = score
		c.generation = generation
		c.mutation = mutation
		c.dnaId = dnaId
		For Local y:Int = 0 Until TCreature.OUTPUTCOUNT
			c.outputs[y] = outputs[y]
			For Local x:Int = 0 Until TCreature.INPUTCOUNT
				c.transform[x, y] = transform[x, y]
			Next
		Next
		For Local x:Int = 0 Until TCreature.INPUTCOUNT
			c.inputs[x] = inputs[x]
		Next
		c.sex = sex
		c.age = age
		Return c
	End Method
	
	Method child:TCreature()
		Local c:TCreature = copy()
		c.sex:*Rnd(0, 1)
		c.age = 0
		c.score = 0
		c.generation:+1
		Return c
	End Method
End Type

Type TDnaStat
	Field counter:Int
	Field totalScore:Int
	
	Method addScore(score:Int)
		counter:+1
		totalScore:+score
	End Method
	
	Method getAvgScore:Float()
		Return (1.0 - (10 / (counter + 10.0))) * totalScore / counter
	End Method
End Type

Function sc(r:Int, g:Int, b:Int)
	SetColor r, g, b
End Function

Function dt(txt:String, x:Int, y:Int)
	SetAlpha 1
	Local r:Int, g:Int, b:Int
	GetColor(r, g, b)
	sc 0, 0, 0
	DrawText txt, x + 2, y + 2
	sc r, g, b
	DrawText txt, x, y
End Function